package entities;


import java.io.Serializable;
import java.util.Date;
import java.util.Set;
import java.util.UUID;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Proxy;
import org.hibernate.annotations.Type;

import com.fasterxml.jackson.annotation.JsonIdentityInfo;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.ObjectIdGenerators;
import com.fasterxml.jackson.annotation.JsonInclude.Include;

@Entity
@Table(name="\"Dataset\"")
@JsonIdentityInfo(generator=ObjectIdGenerators.PropertyGenerator.class, property="id")
public class Dataset implements Serializable,DataEntity {
	
	private static final long serialVersionUID = 3575723814399553259L;

	public Dataset () {}
	
	
	@Id
	@GeneratedValue
    @GenericGenerator(name = "uuid2", strategy = "uuid2")
    @Column(name = "\"ID\"", updatable = false, nullable = false, columnDefinition = "BINARY(16)")
	private UUID id;
	
	@Column(name = "\"Label\"")
	private String label;
	
	
	@ManyToOne(fetch = FetchType.EAGER)
//	@Cascade(value=org.hibernate.annotations.CascadeType.ALL)
    @JoinColumn(name = "\"DMP\"", nullable = true)
	private DMP dmp;
	

	@Column(name = "\"Uri\"")
	private String uri;

	@Type(type="typedefinition.XMLType")
	@Column(name = "\"Properties\"", columnDefinition = "xml", nullable = true)
	private String properties;
	
	
	@ManyToOne(fetch = FetchType.EAGER)
	//@Cascade(value=org.hibernate.annotations.CascadeType.ALL)
    @JoinColumn(name = "\"Profile\"", nullable = true)
	private DatasetProfile profile;
	
	@Type(type="typedefinition.XMLType")
	@Column(name = "\"Reference\"", columnDefinition = "xml", nullable = true)
	private String reference;
	
	@OneToMany(fetch = FetchType.LAZY)
    @JoinTable(name="\"DatasetDataRepository\"", 
    		joinColumns={@JoinColumn(name="\"Dataset\"", referencedColumnName="\"ID\"")},
    	    inverseJoinColumns={@JoinColumn(name="\"DataRepository\"", referencedColumnName="\"ID\"")}
    )  
    private Set<DataRepository> dataRepositories;
	
	
	@OneToMany(fetch = FetchType.LAZY)
    @JoinTable(name="\"DatasetRegistry\"", 
    		joinColumns={@JoinColumn(name="\"Dataset\"", referencedColumnName="\"ID\"")},
    	    inverseJoinColumns={@JoinColumn(name="\"Registry\"", referencedColumnName="\"ID\"")}
    )  
    private Set<Registry> registries;
	
	
	@OneToMany(fetch = FetchType.LAZY)
    @JoinTable(name="\"DatasetService\"", 
    		joinColumns={@JoinColumn(name="\"Dataset\"", referencedColumnName="\"ID\"")},
    	    inverseJoinColumns={@JoinColumn(name="\"Service\"", referencedColumnName="\"ID\"")}
    )  
    private Set<Service> services;
	
	
	@Column(name = "\"Status\"", nullable = false)
	private Short status; 
	
	
	@Column(name = "\"Created\"")
	private Date created = null;
	
	@Column(name = "\"Modified\"")
	private Date modified = new Date();
	
    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "\"Creator\"", nullable = true)
	private UserInfo creator;
	
	@Column(name = "\"Description\"")
	private String description;
	
	
	public String getDescription() {
		return description;
	}
	
	public void setDescription(String description) {
		this.description = description;
	}
	

	public UserInfo getCreator() {
		return creator;
	}

	public void setCreator(UserInfo creator) {
		this.creator = creator;
	}

	public Short getStatus() {
		return status;
	}


	public void setStatus(Short status) {
		this.status = status;
	}


	public Date getCreated() {
		return created;
	}


	public void setCreated(Date created) {
		this.created = created;
	}


	public Date getModified() {
		return modified;
	}


	public void setModified(Date modified) {
		this.modified = modified;
	}


	public Set<Registry> getRegistries() {
		return registries;
	}


	public void setRegistries(Set<Registry> registries) {
		this.registries = registries;
	}


	public Set<Service> getServices() {
		return services;
	}


	public void setServices(Set<Service> services) {
		this.services = services;
	}


	public UUID getId() {
		return id;
	}


	public void setId(UUID id) {
		this.id = id;
	}


	public String getLabel() {
		return label;
	}


	public void setLabel(String label) {
		this.label = label;
	}
	

	public DMP getDmp() {
		return dmp;
	}


	public void setDmp(DMP dmp) {
		this.dmp = dmp;
	}


	public String getUri() {
		return uri;
	}


	public void setUri(String uri) {
		this.uri = uri;
	}


	public String getProperties() {
		return properties;
	}


	public void setProperties(String properties) {
		this.properties = properties;
	}


	public DatasetProfile getProfile() {
		return profile;
	}


	public void setProfile(DatasetProfile profile) {
		this.profile = profile;
	}


	public Set<DataRepository> getDataRepositories() {
		return dataRepositories;
	}


	public void setDataRepositories(Set<DataRepository> dataRepositories) {
		this.dataRepositories = dataRepositories;
	}


	public String getReference() {
		return reference;
	}


	public void setReference(String reference) {
		this.reference = reference;
	}
	
	
	
	
	
}
