package entities;


import java.io.Serializable;
import java.util.Date;
import java.util.Set;
import java.util.UUID;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;

import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Type;

import com.fasterxml.jackson.annotation.JsonIdentityInfo;
import com.fasterxml.jackson.annotation.ObjectIdGenerators;


@Entity
@Table(name="\"DatasetProfile\"")
@JsonIdentityInfo(generator=ObjectIdGenerators.PropertyGenerator.class, property="id")
public class DatasetProfile implements Serializable {
	
	
	private static final long serialVersionUID = 8203086344232867334L;

	public DatasetProfile () {}
	
	
	@Id
	@GeneratedValue
    @GenericGenerator(name = "uuid2", strategy = "uuid2")
    @Column(name = "\"ID\"", updatable = false, nullable = false, columnDefinition = "BINARY(16)")
	private UUID id;
	
	@Column(name = "\"Label\"")
	private String label;
	
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "profile")
	private Set<Dataset> dataset;
	
	
	@OneToOne(fetch = FetchType.EAGER)
//	@Cascade(value=org.hibernate.annotations.CascadeType.ALL)
    @JoinColumn(name = "\"Ruleset\"", nullable = true)
	private DatasetProfileRuleset ruleset;
	
	
	@OneToOne(fetch = FetchType.EAGER)
//	@Cascade(value=org.hibernate.annotations.CascadeType.ALL)
    @JoinColumn(name = "\"Viewstyle\"", nullable = true)
	private DatasetProfileViewstyle viewstyle;
	

	@Type(type="typedefinition.XMLType")
	@Column(name = "\"Definition\"", columnDefinition = "xml", nullable = false)
	private String definition;

	
	@Column(name = "\"Status\"", nullable = false)
	private Short status; 
	
	
	@Column(name = "\"Created\"")
	private Date created;
	
	@Column(name = "\"Modified\"")
	private Date modified = new Date();
	
	@Column(name = "\"Description\"")
	private String description;
	
	
	public String getDescription() {
		return description;
	}


	public void setDescription(String description) {
		this.description = description;
	}
	
	
	public Short getStatus() {
		return status;
	}


	public void setStatus(Short status) {
		this.status = status;
	}


	public Date getCreated() {
		return created;
	}


	public void setCreated(Date created) {
		this.created = created;
	}


	public Date getModified() {
		return modified;
	}


	public void setModified(Date modified) {
		this.modified = modified;
	}
	
	
	
	public UUID getId() {
		return id;
	}

	public void setId(UUID id) {
		this.id = id;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}


	public DatasetProfileRuleset getRuleset() {
		return ruleset;
	}

	public void setRuleset(DatasetProfileRuleset ruleset) {
		this.ruleset = ruleset;
	}
	

	public DatasetProfileViewstyle getViewstyle() {
		return viewstyle;
	}

	public void setViewstyle(DatasetProfileViewstyle viewstyle) {
		this.viewstyle = viewstyle;
	}

	public String getDefinition() {
		return definition;
	}

	public void setDefinition(String definition) {
		this.definition = definition;
	}



	public Set<Dataset> getDataset() {
		return dataset;
	}

	public void setDataset(Set<Dataset> dataset) {
		this.dataset = dataset;
	}

	@Override
	public String toString() {
		return "DatasetProfile [id=" + id + ", label=" + label + ", dataset=" + dataset + ", ruleset=" + ruleset
				+ ", viewstyle=" + viewstyle + ", definition=" + definition + "]";
	}

	
	
}
