package entities.xmlmodels.viewstyledefinition;

import java.util.LinkedList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import entities.xmlmodels.modeldefinition.DatabaseModelDefinition;
import utilities.XmlSerializable;
import utilities.builders.XmlBuilder;

public class FieldSet implements DatabaseViewStyleDefinition,XmlSerializable<FieldSet>{
	private String id;
	private int ordinal;
	private List<Field> fields;

	public List<Field> getFields() {
		return fields;
	}

	public void setFields(List<Field> fields) {
		this.fields = fields;
	}		

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public int getOrdinal() {
		return ordinal;
	}

	public void setOrdinal(int ordinal) {
		this.ordinal = ordinal;
	}

	@Override
	public Element toXml(Document doc) {
		Element fieldSet = doc.createElement("fieldSet");
		fieldSet.setAttribute("id", this.id);
		fieldSet.setAttribute("ordinal", ""+this.ordinal);
		Element fieldsElement = doc.createElement("fields");
		for(Field field : fields){
			fieldsElement.appendChild(field.toXml(doc));
		}
		fieldSet.appendChild(fieldsElement);
		return fieldSet;
	}

	@Override
	public FieldSet fromXml(Element element) {
		this.id = element.getAttribute("id");
		this.ordinal = Integer.parseInt(element.getAttribute("ordinal"));
		this.fields = new LinkedList();
		Element fields = (Element)XmlBuilder.getNodeFromListByTagName(element.getChildNodes(), "fields");

		if(fields!=null){
			NodeList fieldElements = fields.getChildNodes();
			for (int temp = 0; temp < fieldElements.getLength(); temp++) {
				Node fieldElement = fieldElements.item(temp);				
				if (fieldElement.getNodeType() == Node.ELEMENT_NODE) {
					this.fields.add(new Field().fromXml((Element)fieldElement));
				}
			}
		} 
		return this;
	}
	
}
