package models.components.commons.datafield;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import utilities.XmlSerializable;

public class ComboBoxData extends FieldData<ComboBoxData>{
	private class Option implements XmlSerializable<Option>{
		private String label;
		private String value;
		public String getLabel() {
			return label;
		}
		public void setLabel(String label) {
			this.label = label;
		}
		public String getValue() {
			return value;
		}
		public void setValue(String value) {
			this.value = value;
		}
		@Override
		public Element toXml(Document doc) {
			Element option = doc.createElement("option");
			option.setAttribute("label", this.label);
			option.setAttribute("value",this.value);			
			return option;
		}
		@Override
		public Option fromXml(Element item) {
			this.label = item.getAttribute("label");
			this.value = item.getAttribute("value");
			return this;
		}
		
	}
	
	private String type;
	private String url;
	private List<Option> options;
	
	public String getType() {
		return type;
	}
	public void setType(String type) {
		this.type = type;
	}
	public String getUrl() {
		return url;
	}
	public void setUrl(String url) {
		this.url = url;
	}
	public List<Option> getOptions() {
		return options;
	}
	public void setOptions(List<Option> options) {
		this.options = options;
	}
	
	@Override
	public Element toXml(Document doc) {
		Element root = doc.createElement("data");
		root.setAttribute("type", this.type);
		root.setAttribute("url", this.url);
		
		Element element = doc.createElement("options");
		if(this.options!=null){
			for(Option option : this.options){
				element.appendChild(option.toXml(doc));
			}
		}
		root.appendChild(element);
		return root;
	}
	@Override
	public ComboBoxData fromXml(Element item) {
		
		this.url = item.getAttribute("url");
		this.type = item.getAttribute("type");
		this.options = new LinkedList<>();
		Element optionsElement = (Element)item.getElementsByTagName("options").item(0);

		if(optionsElement!=null){
			NodeList optionElements = optionsElement.getChildNodes();
			for (int temp = 0; temp < optionElements.getLength(); temp++) {
				Node optionElement = optionElements.item(temp);				
				if (optionElement.getNodeType() == Node.ELEMENT_NODE) {
					this.options.add(new Option().fromXml((Element)optionElement));
				}
			}
		}
		return this;
	}
	@Override
	public ComboBoxData fromData(Object data) {
		this.options = new LinkedList();

		if(data!=null){
			this.type = (String)((Map<String,Object>)data).get("type");
			this.url = (String)((Map<String,Object>)data).get("url");
			List<Map<String,String>> options = ((Map<String,List<Map<String,String>>>)data).get("options");		
			if(options!=null){
					for(Map<String,String> map : options){
						Option newOption = new Option();
						newOption.setLabel(map.get("label"));
						newOption.setValue(map.get("value"));
						this.options.add(newOption);
					}
			}
		}
		
		return this;
	}
	@Override
	public Object toData() {
		// TODO Auto-generated method stub
		return null;
	}
	
	
}
