package rest.entities;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;

import dao.entities.DatasetProfileDao;
import dao.entities.DatasetProfileRulesetDao;
import dao.entities.DatasetProfileViewstyleDao;
import entities.DatasetProfileViewstyle;
import entities.xmlmodels.modeldefinition.FieldGroup;
import entities.xmlmodels.viewstyledefinition.Section;
import managers.AdminManager;
import models.admin.components.datasetprofile.FieldSet;
import models.admin.composite.DatasetProfile;
import utilities.builders.ModelBuilder;
import utilities.builders.XmlBuilder;
import utilities.helpers.ModelBuilderCollector;

@RestController
@CrossOrigin
public class Admin {

	@Autowired private DatasetProfileDao datasetProfileDao;
	@Autowired private DatasetProfileRulesetDao datasetProfileRulesetDao;
	@Autowired private DatasetProfileViewstyleDao datasetProfileViewstyleDao;

	@Transactional
	@RequestMapping(method = RequestMethod.POST, value = { "/admin/addDmp" },consumes = "application/json", produces="application/json")
	public ResponseEntity<Object> addDmp(@RequestBody DatasetProfile profile){
		try{
			entities.DatasetProfile modelDefinition = AdminManager.generateModelDefinition(profile);
			entities.DatasetProfileViewstyle viewStyleDefinition = AdminManager.generateViewStyleDefinition(profile);

			viewStyleDefinition = datasetProfileViewstyleDao.create(viewStyleDefinition);
			modelDefinition.setViewstyle(viewStyleDefinition);
			datasetProfileDao.create(modelDefinition);

			return ResponseEntity.status(HttpStatus.OK).body(profile);
		}catch(Exception ex){
			ex.printStackTrace();
			return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("{\"reason\":\""+ex.getMessage()+"\"}");
		}
	} 
	
	@Transactional
	@RequestMapping(method = RequestMethod.POST, value = { "/admin/addDmp/{id}" },consumes = "application/json", produces="application/json")
	public ResponseEntity<Object> updateDmp(@PathVariable String id,@RequestBody DatasetProfile profile){
		try{
			entities.DatasetProfile modelDefinition = AdminManager.generateModelDefinition(profile);
			entities.DatasetProfileViewstyle viewStyleDefinition = AdminManager.generateViewStyleDefinition(profile);

			
			entities.DatasetProfile datasetprofile = datasetProfileDao.read(UUID.fromString(id));

			entities.DatasetProfileViewstyle oldviewStyle = datasetProfileViewstyleDao.read((datasetprofile.getViewstyle().getId()));
			oldviewStyle.setDefinition(viewStyleDefinition.getDefinition());
			datasetProfileViewstyleDao.update(oldviewStyle);

			
			//if(!datasetprofile.getDataset().isEmpty())throw new Exception ("Cannot edit a Profile that has Datasets assigned");
			
			datasetprofile.setViewstyle(oldviewStyle);
			datasetprofile.setDefinition(modelDefinition.getDefinition());
			datasetProfileDao.update(datasetprofile);
			
			return ResponseEntity.status(HttpStatus.OK).body(null);
		}catch(Exception ex){
			ex.printStackTrace();
			return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("{\"reason\":\""+ex.getMessage()+"\"}");
		}
	} 
	
	@RequestMapping(method = RequestMethod.GET, value = { "/admin/get/{id}" }, produces="application/json")
	public ResponseEntity<Object> get(@PathVariable String id){
		try{
			entities.DatasetProfile profile = datasetProfileDao.read(UUID.fromString(id));
			models.admin.composite.DatasetProfile datasetprofile = AdminManager.generateDatasetProfileModel(profile);
			return ResponseEntity.status(HttpStatus.OK).body(datasetprofile);
		}catch(Exception ex){
			ex.printStackTrace();
			return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("{\"reason\":\""+ex.getMessage()+"\"}");
		}
	}
	
}
